// Generated using webpack-cli https://github.com/webpack/webpack-cli

// npm publish  --registry https://registry.npmjs.org
const isProduction = process.env.NODE_ENV == "production";

const { VueLoaderPlugin } = require('vue-loader')
const path = require("path");
// const HtmlWebpackPlugin = require("html-webpack-plugin");
const MiniCssExtractPlugin = require("mini-css-extract-plugin");
// const VueLoaderPlugin = require('vue-loader/lib/plugin')
const webpack = require('webpack');


function resolve(dir) {
  return path.join(__dirname, dir)
}

const stylesHandler = isProduction ? MiniCssExtractPlugin.loader : "style-loader";

const config = {
  entry: "./packages/index.ts",//"./examples/index.ts",
  experiments: {
    outputModule: true,
  },
  output: {
    module: true, 
    filename: 'davidwingvue3.min.js',
    // publicPath: process.env.NODE_ENV === "development" ? "/" : "/emacroh5lib/",
    path: path.resolve(__dirname, "dist"),
    library: {
      // do not specify a `name` here
      type: 'module'
    },
    environment: { 
      module: true,
      // dynamicImport: true,   // Note you need to enable `dynamicImport ` here
    },
  },
  // externalsType: "import",
  optimization: { sideEffects: true },
  externals: {  // 排除打包,瘦身,以及解决依赖冲突
    'vue': 'vue'
  },
  plugins: [
    new VueLoaderPlugin(),
    // new HtmlWebpackPlugin({
    //   template: "index.html",
    // }),
    new webpack.ProvidePlugin({
      // echarts: 'echarts',
      // THREE: 'three',
      // jQuery: "jquery",
      // $: "jquery"
    })

    // Add your plugins here
    // Learn more about plugins from https://webpack.js.org/configuration/plugins/
  ],
  module: {
    rules: [
      {
        test: /\.(vue)$/,
        use: [
          {
            loader: 'vue-loader',
            options: {
              esModule: true
            }
          }
        ]
      },
      {
        test: /\.(ts|tsx)$/i,
        loader: "ts-loader",
        options: {
          appendTsSuffixTo: [/\.(vue)$/],
          transpileOnly: true //只进行语法转换, 不进行类型校验, 提高构建速度
        },
        exclude: ["/node_modules/"],
      },
      {
        test: /\.less$/i,
        use: ['style-loader', 'css-loader', 'less-loader'],
        // use: [stylesHandler, "css-loader", "postcss-loader", "less-loader"], // 会丢失样式
      },
      {
        test: /\.css$/i,
        use: [stylesHandler, "css-loader", "postcss-loader"],
      },
      {
        test: /\.(eot|svg|ttf|woff|woff2|png|jpg|gif|ico)$/i,
        type: "asset/inline"
      },


      // Add your rules for custom modules here
      // Learn more about loaders from https://webpack.js.org/loaders/
    ],
  },
  resolve: {
    alias: {
      '@': resolve('examples')
    },
    fallback: {
      crypto: false,
      // fs: false
    },
    extensions: ['.js', '.jsx', '.ts', '.tsx', '.vue', '.json'],
  }
};

module.exports = () => {
  if (isProduction) {
    config.mode = "production";

    config.plugins.push(new MiniCssExtractPlugin());
    // config.plugins.push(new VueLoaderPlugin());
  } else {
    config.mode = "development";
  }

  // 打包文件大小配置
  config.performance = {
    maxEntrypointSize: 10 * 1024 * 1024,
    maxAssetSize: 10 * 1024 * 1024
  }

  return config;
};
